"""JARVIS task mixin."""

from datetime import datetime

import pytz
from aiohttp import ClientSession
from croniter import croniter
from interactions.models.internal.tasks.task import Task
from interactions.models.internal.tasks.triggers import BaseTrigger, IntervalTrigger
from jarvis_core.db import models

from jarvis.utils import cleanup_user


class CronTrigger(BaseTrigger):
    """
    Trigger the task based on a cron schedule.

    Attributes:
        schedule str: The cron schedule, use https://crontab.guru for help

    """

    def __init__(self, schedule: str) -> None:
        self.schedule = schedule

    def next_fire(self) -> datetime | None:
        return croniter(self.schedule, datetime.now(tz=pytz.utc)).next(datetime)


class TaskMixin:
    @Task.create(IntervalTrigger(minutes=1))
    async def _update_domains(self) -> None:
        async with ClientSession(headers={"X-Identity": "Discord: zevaryx#5779"}) as session:
            response = await session.get("https://phish.sinking.yachts/v2/recent/60")
            response.raise_for_status()
            data = await response.json()

        if len(data) == 0:
            return
        self.logger.debug(f"Found {len(data)} changes to phishing domains")

        add = 0
        sub = 0

        for update in data:
            if update["type"] == "add":
                for domain in update["domains"]:
                    if domain not in self.phishing_domains:
                        add += 1
                        self.phishing_domains.append(domain)
            elif update["type"] == "delete":
                for domain in update["domains"]:
                    if domain in self.phishing_domains:
                        sub -= 1
                        self.phishing_domains.remove(domain)
        self.logger.info(f"[antiphish] {add} additions, {sub} removals")

    @Task.create(IntervalTrigger(minutes=30))
    async def _update_currencies(self) -> None:
        await self.erapi.update_async()

    @Task.create(CronTrigger("0 0 1 * *"))
    async def _cleanup(self) -> None:  # noqa: C901
        self.logger.debug("Getting all unique guild, channel, and user IDs")
        guild_ids = []
        channel_ids = []
        user_ids = []
        for model in models.all_models:
            # Simple IDs
            if hasattr(model, "guild"):
                ids = await model.distinct(model.guild)
                guild_ids += [x for x in ids if x not in guild_ids]
            if hasattr(model, "channel"):
                ids = await model.distinct(model.channel)
                channel_ids += [x for x in ids if x not in channel_ids]
            if hasattr(model, "admin"):
                ids = await model.distinct(model.admin)
                user_ids += [x for x in ids if x not in user_ids]
            if hasattr(model, "user"):
                ids = await model.distinct(model.user)
                user_ids += [x for x in ids if x not in user_ids]
            if hasattr(model, "creator"):
                ids = await model.distinct(model.creator)
                user_ids += [x for x in ids if x not in user_ids]
            if hasattr(model, "editor"):
                ids = await model.distinct(model.editor)
                user_ids += [x for x in ids if x and x not in user_ids]
            # Complex IDs
            if hasattr(model, "users"):
                async for result in model.find():
                    user_ids += [x for x in result.users if x not in user_ids]

        self.logger.debug("Cleaning up missing guilds")
        for guild_id in guild_ids:
            guild = await self.fetch_guild(guild_id)
            if not guild:
                self.logger.info(f"Guild {guild_id} not found, deleting references")
                for model in models.all_models:
                    if hasattr(model, "guild"):
                        await model.find(model.guild == guild_id).delete()
            else:
                await guild.chunk()

        self.logger.debug("Cleaning up missing channels")
        for channel_id in channel_ids:
            channel = await self.fetch_channel(channel_id)
            if not channel:
                self.logger.info(f"Channel {channel_id} not found, deleting references")
                for model in models.all_models:
                    if hasattr(model, "channel"):
                        await model.find(model.channel == channel_id).delete()

        self.logger.debug("Cleaning up missing users")
        for user_id in user_ids:
            user = await self.fetch_user(user_id)
            if not user:
                await cleanup_user(self, user_id)
            elif len(user.mutual_guilds) == 0:
                await cleanup_user(self, user_id)

        self.logger.debug("Done with cleanup! Running again in 1 month")
