"""JARVIS data management ext."""

import asyncio
import logging

from interactions import Client, Extension, SlashContext
from interactions.models.internal.application_commands import SlashCommand
from interactions.models.discord.components import ActionRow, Button, ButtonStyle

from jarvis.utils import cleanup_user

WARNING_MESSAGE = """***Are you sure you want to delete your data?***

If you delete your data, you will:

- Lose any reminders that are set
- Lose all custom settings for your account set in JARVIS
- All admin tasks and created tags will be re-assigned to JARVIS

Once done, there is no way to go back! Please think carefully before proceeding.

This interaction will time out in 60 seconds
"""


class DataCog(Extension):
    """
    Data functions for JARVIS

    Data deletion and requests are made here
    """

    def __init__(self, bot: Client):
        self.bot = bot
        self.logger = logging.getLogger(__name__)

    data = SlashCommand(name="data", description="Data commands")

    @data.subcommand(sub_cmd_name="usage", sub_cmd_description="View data usage information")
    async def _data_usage(self, ctx: SlashContext) -> None:
        await ctx.send("Please review our [Privacy Policy](https://s.zevs.me/jarvis-privacy)")

    @data.subcommand(sub_cmd_name="delete", sub_cmd_description="Delete my data")
    async def _data_delete(self, ctx: SlashContext) -> None:
        no = Button(
            style=ButtonStyle.RED,
            label="Nevermind, I changed my mind",
            custom_id="delete_data||no",
        )
        yes = Button(
            style=ButtonStyle.GREEN,
            label="Yes, delete my data",
            custom_id="delete_data||yes",
        )
        components = [ActionRow(no, yes)]
        message = await ctx.send(WARNING_MESSAGE, components=components)
        try:
            response = await self.bot.wait_for_component(
                messages=message,
                check=lambda x: ctx.author.id == x.ctx.author.id,
                timeout=60,
            )
        except asyncio.TimeoutError:
            for row in components:
                for component in row.components:
                    component.disabled = True
            await message.edit(
                content=message.content + "\n\nInteraction timed out",
                components=components,
            )
            return
        try:
            if response.ctx.custom_id.split("||")[-1] == "yes":
                for row in components:
                    for component in row.components:
                        component.disabled = True
                await response.ctx.edit_origin(
                    components=components,
                )
                message = await message.reply(
                    "<a:loading:1014424332648325150> Deleting your data, this may take a minute..."
                )
                try:
                    await cleanup_user(self.bot, ctx.author.id)
                    await message.edit(content="✅ Your data has been deleted!")
                except Exception as e:
                    await message.edit(
                        content="❌ There was an error. Please report it to the help server, or DM `@zevaryx` for help"
                    )
                    self.logger.error(f"Encountered error while deleting data: {e}", exc_info=True)

            else:
                for row in components:
                    for component in row.components:
                        component.disabled = True
                await response.ctx.edit_origin(
                    content=message.content + "\n\nInteraction cancelled",
                    components=components,
                )
        except Exception as e:
            self.logger.error(f"Encountered error: {e}", exc_info=True)


def setup(bot):
    DataCog(bot)
