"""JARVIS Utility Functions."""

from datetime import datetime, timezone
from pkgutil import iter_modules
from typing import Optional

import pytz
from beanie.operators import Set
from interactions import Client
from interactions.models.discord.embed import Embed, EmbedField
from interactions.models.discord.guild import AuditLogEntry
from interactions.models.discord.user import Member
from jarvis_core.db import models

from jarvis.branding import PRIMARY_COLOR


def build_embed(
    title: str,
    description: str,
    fields: list,
    color: str = PRIMARY_COLOR,
    timestamp: Optional[datetime] = None,
    **kwargs: dict,
) -> Embed:
    """Embed builder utility function."""
    if not timestamp:
        timestamp = datetime.now(tz=timezone.utc)
    embed = Embed(
        title=title,
        description=description,
        color=color,
        timestamp=timestamp,
        **kwargs,
    )
    for field in fields:
        embed.add_field(**field.to_dict())
    return embed


def modlog_embed(
    member: Member,
    admin: Member,
    title: str,
    desc: str,
    log: Optional[AuditLogEntry] = None,
    reason: Optional[str] = None,
) -> Embed:
    """Get modlog embed."""
    fields = [
        EmbedField(
            name="Moderator",
            value=f"{admin.mention} ({admin.username})",
        ),
    ]
    if log and log.reason:
        reason = reason or log.reason
    fields.append(EmbedField(name="Reason", value=reason, inline=False))
    embed = build_embed(
        title=title,
        description=desc,
        color="#fc9e3f",
        fields=fields,
        timestamp=log.created_at,
    )
    embed.set_author(name=f"{member.username}", icon_url=member.display_avatar.url)
    embed.set_footer(text=f"{member.username} | {member.id}")
    return embed


def get_extensions(path: str) -> list:
    """Get JARVIS cogs."""
    vals = [x.name for x in iter_modules(path)]
    return [f"jarvis.cogs.{x}" for x in vals]


async def cleanup_user(bot: Client, user_id: int):
    for model in models.all_models:
        if hasattr(model, "admin"):
            await model.find(model.admin == user_id).update(Set({model.admin: bot.user.id}))
        if hasattr(model, "user"):
            await model.find(model.user == user_id).delete()
        if hasattr(model, "creator"):
            await model.find(model.creator == user_id).update(
                Set(
                    {
                        model.creator: bot.user.id,
                        model.editor: bot.user.id,
                        model.edited_at: datetime.now(tz=pytz.utc),
                    }
                )
            )
        if hasattr(model, "editor"):
            await model.find(model.editor == user_id).update(
                Set(
                    {
                        model.editor: bot.user.id,
                        model.edited_at: datetime.now(tz=pytz.utc),
                    }
                )
            )
        if hasattr(model, "users"):
            async for result in model.find():
                result.users.remove(user_id)
                await result.save()
