"""JARVIS update handler."""

from dataclasses import dataclass
from importlib import import_module
from inspect import getmembers, isclass
from pkgutil import iter_modules
from types import FunctionType, ModuleType
from typing import Any, Callable, Dict, List

from interactions.client.utils.misc_utils import find_all
from interactions.models.internal.application_commands import SlashCommand
from interactions.models.internal.extension import Extension
from rich.table import Table

import jarvis.cogs


@dataclass
class UpdateResult:
    """JARVIS update result."""

    old_hash: str
    new_hash: str
    table: Table
    added: List[str]
    removed: List[str]
    changed: List[str]
    lines: Dict[str, int]


def get_all_commands(module: ModuleType = jarvis.cogs) -> Dict[str, Callable]:
    """Get all SlashCommands from a specified module."""
    commands = {}

    def validate_ires(entry: Any) -> bool:
        return isclass(entry) and issubclass(entry, Extension) and entry is not Extension

    def validate_cog(cog: FunctionType) -> bool:
        return isinstance(cog, SlashCommand)

    for item in iter_modules(module.__path__):
        new_module = import_module(f"{module.__name__}.{item.name}")
        if item.ispkg:
            if cmds := get_all_commands(new_module):
                commands.update(cmds)
        else:
            inspect_result = getmembers(new_module)
            cogs = []
            for _, val in inspect_result:
                if validate_ires(val):
                    cogs.append(val)
            for cog in cogs:
                values = cog.__dict__.values()
                commands[cog.__module__] = find_all(lambda x: isinstance(x, SlashCommand), values)
    return {k: v for k, v in commands.items() if v}
